""" ClinicalOmicsDB: Bridging the gap between next-generation clinical omics data and machine learning
Reference: Chang In Moon, Byron Jia, Bing Zhang
Paper link: TBD
Code author: Chang In Moon (moonchangin@gmail.com)
-----------------------------
USAGE
$ python3 download_link.py
- interactive python script that ask several query which give a table with downloadble link and an option to download all of them

"""


import re
import io
import sys
import inquirer
from database_download import download_sheet
import pandas as pd
from datetime import datetime
from file_download import download_text as _download_text

# Disease inquiry
questions_1 = [
  inquirer.Checkbox('disease',
                    message="What type of disease type you would like to download? (Left and right arrow to select multiple)",
                    choices=['Breast', 'Colorectal', 'Leukemia', 'Lung', 'Melanoma', 'Myeloma', 'Ovarian'],
                    default='Breast'
                    ),
]
disease_type = inquirer.prompt(questions_1)
# if breast cancer is selected promt breast cancer subtype (11/14/22 continue...)
if "Breast" in  disease_type["disease"][:]:
    questions_1_1 = [
    inquirer.Checkbox('breast_subtype',
                    message="What breast cancer subtype you would like to download? (Left and right arrow to select multiple)",
                    choices=['Combined', 'HR+', 'HER2+', 'TNBC'],
                    default='Combined'
                    ),]
    breast_subtype = inquirer.prompt(questions_1_1)

# Treatment inquiry
questions_2 = [
  inquirer.Checkbox('treatment',
                    message="What treatment type you would like to download? (Left and right arrow to select multiple)",
                    choices=['Cytotoxic', 'Targeted', 'Immunotherapy'],
                    default='Cytotoxic'
                    ),
]
treatment_type = inquirer.prompt(questions_2)

# Download inquiry
questions_3 = [
  inquirer.List('download',
                    message="Would you like to download the selected data to your current directory?",
                    choices=['Yes (it may take a few minutes depending on network condition)', 'No (use the link in the excel to manually download)'],
                    default='No (use the download link in the CSV excel to manually download)'
                    ),
]
download = inquirer.prompt(questions_3)

# Based on inquiry filter out the data
main_db = download_sheet() 
# merge dictionary as one
if 'breast_subtype' in globals():
  filter = {**disease_type,**breast_subtype,**treatment_type}
else:
  filter = {**disease_type,**treatment_type}
print("Query added by user: " + str(filter))
# filter database
##Start with array of all True
ind = [True] * len(main_db)
##Loop through filters, updating index
for col, vals in filter.items():
    ind = ind & (main_db[col].isin(vals))
##Return filtered dataframe
output = main_db[ind]
if output.empty:
  sys.exit('[ERROR] No data was found from your query :( \n[ERROR] Please try again with more options checked!')
output.to_csv("clindb_download_queue_" + datetime.now().strftime("%H:%M:%S") + ".csv")

# begin the download if the user said Yes on the final question
if 'Yes (it may take a few minutes depending on network condition)' in download.values():
    print("[DOWNLOADING] This may take a few minutes depending on network condition. Thank you for you for being patient!")
    list_link = output[['Series','download_link']]
    list_link = list_link.reset_index()
    for file, row in list_link.iterrows():
        text = _download_text(row['download_link'])
        tmp = pd.read_csv(io.StringIO(text), header=0)
        tmp.to_csv(row['Series'])
        print("[LOG] " + str(row['Series']) + " download complete!")


""" ClinicalOmicsDB: Bridging the gap between next-generation clinical omics data and machine learning
Reference: Chang In Moon, Byron Jia, Bing Zhang
Paper link: TBD
Code author: Chang In Moon (moonchangin@gmail.com)
-----------------------------

USAGE:
$ python3 main_single_biomarker.py --gene NF1 --output NF1_clindb_significance.csv

output: list of database infomation with wilcoxon p-value across databases
"""
# Necessary packages
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import argparse
from datetime import datetime
import numpy as np
import pandas as pd
import os
from scipy.stats import mannwhitneyu

from database_download import download, biomarker_download_sheet


# define and edit parameters for supervised models
def main(args):
  """Main function for wilcoxon statistical testing.
  Returns:
  output: list of database infomation with wilcoxon p-value across databases
  """
  collection = biomarker_download_sheet()
  db_name = collection.iloc[:, 0]
  output_df = []
  for f in db_name:
      print("Testing " + f)
      db_info = collection.loc[collection['Series'] == f]
      url = db_info.iloc[0,6]
      
      temp_df = download(url)      
      temp_df = temp_df.dropna(axis='columns') # Remove genes with missing values
      y = temp_df.iloc[:, 1] # Response is on 2nd column
      try:
        # group_a = non_responders
        group_a = temp_df[temp_df.iloc[:, 1] == 0]
        # group_a = group_a.iloc[:, 2:] # expression data only
        gene =  args.gene
        group_a = group_a.loc[:, gene]
        
        # group_b = responders
        group_b = temp_df[temp_df.iloc[:, 1] == 1]
        # group_b = group_b.iloc[:, 2:] # expression data only
        group_b = group_b.loc[:, gene]
      except Exception:
        print("[WARNING] Skipping " + f + " because gene symbol " + gene + " cannot be found in this dataset")
        break # do not print or save results if the dataset do not have the gene symbol
      # wilcoxon test
      _ , p_value = mannwhitneyu(group_a, group_b, alternative="two-sided")
      # parse file name
      database_name = f[10:]
      database_name = database_name[:-4]
      test_eval = {'dataset': database_name,
        'sample_size': temp_df.shape[0],
        'feature_size': temp_df.shape[1] - 1,
        'responder_log_ratio': round(abs(np.log(y.sum()/(len(y)-y.sum()))),2),
        'platform': db_info.iloc[0,5],
        'cancer_type': db_info.iloc[0,1],
        'treatment_type': db_info.iloc[0,3],
        'response_evaluation': db_info.iloc[0,4],
        'wilcoxon_p_val': p_value
        }
      print(test_eval)
      output_df.append(test_eval)
  # save your final results
  df2 = pd.DataFrame(output_df)
  df2_sorted = df2.sort_values(by='wilcoxon_p_val', ascending=True)
  df2_sorted.to_csv(args.output)
  
if __name__ == '__main__':
  # Inputs for the main function
  parser = argparse.ArgumentParser()
  parser.add_argument(
      '--gene',
      help='Single gene symbol input',
      required=True)  
  parser.add_argument(
      '--output',
      help='name of the output file (default: clinicalomicsdb_output_[CURRENT_TIME].csv)',
      default="clinicalomicsdb_output_" + datetime.now().strftime("%H:%M:%S") + ".csv",
      type=str)
  
  args = parser.parse_args() 
  # Calls main function  
  main(args)


""" ClinicalOmicsDB: Bridging the gap between next-generation clinical omics data and machine learning
Reference: Chang In Moon, Byron Jia, Bing Zhang
Paper link: TBD
Code author: Chang In Moon (moonchangin@gmail.com)
-----------------------------

USAGE:
$ python3 main_clindb_subtype_biomarkers.py --model_list logit --mc_split 10 --biomarker MKI67 AURKA BIRC5 CCNB1 MYBL2 ERBB2 GRB7 ESR1 PGR BCL2 SCUBE2 MMP11 CTSL2 GSTM1 CD68 BAG1 ACTB GAPDH RPLPO GUS TFRC

output: list of database auroc or acc performance across transcriptomics databases labeled in gene symbols
"""
# Necessary packages
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import argparse
#from importlib_metadata import files
from datetime import datetime
import numpy as np
import pandas as pd
import os

from database_download import download, download_sheet, biomarker_download_sheet, HR_dataset_link, HER2_dataset_link, TNBC_dataset_link
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import train_test_split
from preset_supervised_models_biomarkers import logit, xgb_model, mlp, svm, knn, rf, custom
from utils import perf_metric

# define and edit parameters for supervised models
def supervised_model_training (x_train, y_train, x_test,
                               y_test, model_name, metric):
  """Train supervised learning models and report the results.
  list of parameters will be definded in the function
  Args:
    - x_train, y_train: training dataset
    - x_test, y_test: testing dataset
    - model_name: logit, xgb_model, mlp, svm, knn, rf, custom
    - metric: acc or auc

  Returns:
    - performance: prediction performance
  """

  # Train supervised model w/ specified hyperparameters
  # Logistic regression
  if model_name == 'logit':
    log_param = dict()
    log_param['classifier__l1_ratio'] = np.arange(0.1,1.1,0.2)
    log_param['classifier__C'] = np.logspace(-3,3,5)
    y_test_hat = logit(x_train, y_train, x_test, log_param)
  # XGBoost
  elif model_name == 'xgb_model':
    xg_param = dict()
    xg_param['classifier__max_depth'] = [4, 6, 8, 10]
    xg_param['classifier__n_estimators'] = range(100, 400, 100)
    xg_param['classifier__use_label_encoder'] = [False]
    xg_param['classifier__eval_metric'] = ["logloss"]
    y_test_hat = xgb_model(x_train, y_train, x_test, xg_param)
  # MLP
  elif model_name == 'mlp':
    mlp_param = dict()
    mlp_param['classifier__hidden_layer_sizes'] = [(10,5)]
    mlp_param['classifier__activation'] = ['relu']
    mlp_param['classifier__solver'] = ['adam']
    mlp_param['classifier__max_iter'] = [5000]
    mlp_param['classifier__alpha'] = [0.0001, 0.05]
    mlp_param['classifier__learning_rate'] = ['constant','adaptive']
    y_test_hat = mlp(x_train, y_train, x_test, mlp_param)
  # SVM
  elif model_name == 'svm':
    svm_param = dict()
    svm_param['classifier__C'] = [0.001, 0.01, 0.1, 1, 10, 100]
    svm_param['classifier__gamma'] = [0.001, 0.01, 0.1, 1, 10, 100]
    y_test_hat = svm(x_train, y_train, x_test, svm_param)
  # KNN
  elif model_name == 'knn':
    knn_param = dict()
    knn_param['classifier__n_neighbors'] = range(1, 10, 1)
    knn_param['classifier__weights'] = ['uniform','distance']
    knn_param['classifier__p'] = [1,2]
    y_test_hat = knn(x_train, y_train, x_test, knn_param)
  # Random Forest
  elif model_name == 'rf':
    rf_param = dict()
    rf_param['classifier__max_depth'] = [4, 6, 8, 10]
    rf_param['classifier__n_estimators'] = range(100, 400, 100)
    y_test_hat = rf(x_train, y_train, x_test, rf_param)
  # User customized ML model
  elif model_name == 'custom':
    custom_param = dict() # user must add in required parameters here
    y_test_hat = custom(x_train, y_train, x_test, custom_param) # user must create a customized function in "preset_supervised_models.py"
  # Report the performance
  performance = perf_metric(metric, y_test, y_test_hat)
  return performance

def main(args):
  """Main function for benchmarking.
  
  Args:
    - mc_split: Number of monte carlo cross validation splits [default: 50]
    - model_name: List of ML model name e.g. [mlp, logit, or xgboost]
    - output: name of the output file [default: clinicalomicsdb_output.csv]
    - metric: acc or auc [default: auc]
    - user_data: data (X) input the user would like to independantly train on [optional;
     1st column: sample_name; 2nd column: response, 3rd+ column: other biomarkers]
    - user_label: label (y) input the user [optional; must match with data X samples]

  Returns:
    - a csv file with all performances of the models (supervised; semi-supervised; customized models etc...)
  """
  if args.subset == "all":
    collection = biomarker_download_sheet()
    collection = collection[(collection["breast_subtype"] == "Combined") | (collection["breast_subtype"] == "not_breast")]
  elif args.subset == "BRCA_HR":
    collection = biomarker_download_sheet()
    collection = collection[collection["breast_subtype"] == "HR+"]
  elif args.subset == "BRCA_HER2":
    collection = biomarker_download_sheet()
    collection = collection[collection["breast_subtype"] == "HER2+"]
  elif args.subset == "BRCA_TNBC":
    collection = biomarker_download_sheet()
    collection = collection[collection["breast_subtype"] == "TNBC"]
  db_name = collection.iloc[:, 0]
  output_df = []
  # user defined data will be analyzed first if specified
  if args.user_data != None:
      print("Training " + args.user_data)
      # accept data and labels that is in a csv file
      data = pd.read_csv(args.user_data, header="infer") 
      X = data.iloc[:, 2:] # expression data only
      # FEATURE SELECTION BY BIOMARKER
      selected_features =  args.biomarker
      X = X.loc[:, selected_features]
      y = data.iloc[:, 1] # Response is on 2nd column
      model_sets = args.model_list
      seed = 1
      while seed <= args.mc_split:
          # perform a stratified monte carlo cross validation (Here we will leave 20% of testing for final model evalutation)
          X_train, X_test, y_train, y_test = train_test_split(X, y, train_size=0.8, random_state=seed, stratify=y)
          for m_it in range(len(model_sets)):
              model_name = model_sets[m_it]
              tmp = supervised_model_training(X_train, y_train, X_test,
                                                      y_test, model_name, metric = args.metric)
              test_eval = {'dataset': args.user_data, 'model': model_sets[m_it], 'auroc':tmp}
              print(test_eval)
              output_df.append(test_eval)
          seed += 1
  # rest of the model method will be performed on all datasets
  for f in db_name:
      print("Training " + f)
      db_info = collection.loc[collection['Series'] == f]
      url = db_info.iloc[0,6]
      temp_df = download(url)      
      temp_df = temp_df.dropna(axis='columns') # Remove genes with missing values
      X = temp_df.iloc[:, 2:] # expression data only
      # FEATURE SELECTION BY BIOMARKER
      selected_features =  args.biomarker
      # X = X.loc[:, selected_features]
      X = X.loc[:, X.columns.isin(selected_features)]
      y = temp_df.iloc[:, 1] # Response is on 2nd column
      
      # Define supervised model & inputs
      model_sets = args.model_list
      seed = 1
      while seed <= args.mc_split:
          # perform a stratified monte carlo cross validation (Here we will leave 20% of testing for final model evalutation)
          X_train, X_test, y_train, y_test = train_test_split(X, y, train_size=0.8, random_state=seed, stratify=y)
          for m_it in range(len(model_sets)):
              model_name = model_sets[m_it]
              try:
                tmp = supervised_model_training(X_train, y_train, X_test,
                                                      y_test, model_name, metric = args.metric)
              except Exception:
                print("Skipping " + f + " due to either small sample size or imbalanced labels")
                break # do not print or save results if the database have imbalanced labels or small sample size
              # parse file name
              database_name = f[10:]
              database_name = database_name[:-4]
              test_eval = {'dataset': database_name,
               'model': model_sets[m_it],
                'auroc': tmp,
                 'sample_size': X.shape[0],
                 'feature_size': X.shape[1],
                 'responder_log_ratio': round(abs(np.log(y.sum()/(len(y)-y.sum()))),2),
                 'platform': db_info.iloc[0,5],
                 'cancer_type': db_info.iloc[0,1],
                 'treatment_type': db_info.iloc[0,3],
                 'response_evaluation': db_info.iloc[0,4]
                 }
              print(test_eval)
              output_df.append(test_eval)
          seed += 1
  # save your final results
  df2 = pd.DataFrame(output_df)
  df2.to_csv(args.output)
  
if __name__ == '__main__':
  # Inputs for the main function
  parser = argparse.ArgumentParser()
  parser.add_argument(
      '-l',
      '--model_list',
      help='list of ML model name (e.g. logit xgboost custom etc...)',
      nargs='+',
      required=True)  
  parser.add_argument(
      '--biomarker',
      help='Specify your biomarker to interest ([PAM50 signiture genes]: CCND1 CDH1 CDH3 CDK4 ESR1 FOXA1 GATA3 KRT5 KRT17 KRT18 KRT19 KRT20 KRT7 KRT8 MYC PGR TP53)',
      nargs='+',
      required=True)  
  parser.add_argument(
      '--mc_split',
      help='number of monte carlo cross validation per database (default: 30)',
      default=30,
      type=int)
  parser.add_argument(
      '--output',
      help='name of the output file (default: clinicalomicsdb_output_[CURRENT_TIME].csv)',
      default="clinicalomicsdb_output_" + datetime.now().strftime("%H:%M:%S") + ".csv",
      type=str)
  parser.add_argument(
      '--metric',
      help='acc or auc (default: auc)',
      default="auc",
      type=str)
  parser.add_argument(
      '--user_data',
      help='data(X,y) input the user would like to independantly train on (OPTIONAL) (FORMAT:1st column: sample_name; 2nd column: response, 3rd+ column: other biomarkers)',
      default=None,
      type=str)
  parser.add_argument(
      '--subset',
      help='select particular subset of cohort for machine learning analysis (default: all) [available subset: BRCA_HR, BRCA_HER2, BRCA_TNBC]',
      default="all",
      type=str)  
  args = parser.parse_args() 
  # Calls main function  
  main(args)


""" ClinicalOmicsDB: Bridging the gap between next-generation clinical omics data and machine learning
Reference: Chang In Moon, Byron Jia, Bing Zhang
Paper link: TBD
Code author: Chang In Moon (moonchangin@gmail.com)
-----------------------------

preset_supervised_models.py
- Collection of preset supervised models and return predicted labels for testing dataset
- This include a pipeline that can use preprocessing and feature selection as a hyperparameter
- Users may create a custom function in order to modify and test out their modified algorithms

(0) custom: user defined classifier

(1) logit: logistic regression
Source: Sammut, SJ Nature (2021)

(2) xgb_model: XGBoost model
Source: Shi, Z Molecular cellular proteomics (2021)

(3) mlp: multi-layer perceptron
Source: https://michael-fuchs-python.netlify.app/2021/02/03/nn-multi-layer-perceptron-classifier-mlpclassifier/

(4) svm: support vector machine classifier
Source: Shi, Z Molecular cellular proteomics (2021)

(5) knn: k-nearest neighbors classifier
Source: https://www.kaggle.com/code/abhinavdevarajan/breast-cancer-analysis-with-hyper-parameter-tuning

(6) rf: random forest classifier
Source: Shi, Z Molecular cellular proteomics (2021)

"""

# Necessary packages
from sklearn.linear_model import LogisticRegression # Logistic Reg (L2)
import xgboost as xgb #xgboost
from sklearn.neural_network import MLPClassifier
from sklearn.svm import SVC # SVM
from sklearn.neighbors import KNeighborsClassifier # KNN
from sklearn.ensemble import RandomForestClassifier # Random Forest
from sklearn.tree import DecisionTreeClassifier # decision tree for custom algorithm
from sklearn.feature_selection import SelectKBest , VarianceThreshold
from sklearn.model_selection import GridSearchCV 
from sklearn.pipeline import Pipeline

# %%
def custom(x_train, y_train, x_test, parameters=None):
  """Custom Classifier Specified by the user
  Feel free to customize this function that can return a y_test_hat (probability) at the end of the function.
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization
    
  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', DecisionTreeClassifier())
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 8, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None:    
      model = DecisionTreeClassifier()
      model.fit(x_train, y_train)  
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test)
  return y_test_hat

#%% 
def logit(x_train, y_train, x_test, parameters=None):
  """Logistic Regression.
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization

  Returns:
    - y_test_hat: predicted values for x_test
  """
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', LogisticRegression(penalty='elasticnet', solver='saga', max_iter=10000, class_weight=True))
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 8, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None:    
      model = LogisticRegression()
      model.fit(x_train, y_train)
  print(model.best_params_)
  y_test_hat = model.predict_proba(x_test) 
  return y_test_hat
#%% 
def xgb_model(x_train, y_train, x_test, parameters=None):
  """XGBoost.
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization

  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', xgb.XGBClassifier(verbosity = 0, silent=True, n_jobs = 4))
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 4, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None:    
      model = xgb.XGBClassifier(verbosity = 0, silent=True, n_jobs = 8)
      model.fit(x_train, y_train)
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test) 
  return y_test_hat
#%% 
def mlp(x_train, y_train, x_test, parameters=None):
  """multi-layer perceptrons
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization

  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', MLPClassifier())
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 8, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None: 
      model = MLPClassifier()   
      model.fit(x_train, y_train)
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test)
  return y_test_hat
#%% 
def svm(x_train, y_train, x_test, parameters=None):
  """support vector machine classifier
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization
    
  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', SVC(probability=True))
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 8, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None: 
      model = SVC(probability = True)
      model.fit(x_train, y_train)
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test)
  return y_test_hat
# %%
def knn(x_train, y_train, x_test, parameters=None):
  """k-nearest neighbors classifier
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization
    
  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', KNeighborsClassifier())
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 8, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None:    
      model = KNeighborsClassifier()
      model.fit(x_train, y_train)
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test)
  return y_test_hat
# %%
def rf(x_train, y_train, x_test, parameters=None):
  """Random Forest Classifier
  
  Args: 
    - x_train, y_train: training dataset
    - x_test: testing feature
    - parameters: hyperparameter for optimization
    
  Returns:
    - y_test_hat: predicted values for x_test
  """  
  if parameters != None:
      pipe = Pipeline([
      ('filter', VarianceThreshold()),
      ('kbest', SelectKBest()),
      ('classifier', RandomForestClassifier(n_jobs = 4))
      ])
      model = GridSearchCV(pipe, parameters, scoring='roc_auc', cv=5, refit=True, n_jobs = 4, error_score=0)
      model.fit(x_train, y_train)
  elif parameters == None:    
      model = RandomForestClassifier(n_jobs = 4)
      model.fit(x_train, y_train)  
  print(model.best_params_) 
  y_test_hat = model.predict_proba(x_test) 
  return y_test_hat


""" ClinicalOmicsDB: Bridging the gap between next-generation clinical omics data and machine learning
Reference: Chang In Moon, Byron Jia, Bing Zhang
Paper link: TBD
Code author: Chang In Moon (moonchangin@gmail.com)
-----------------------------
"""
__version__ = '0.4.6'

import io
import pandas as pd
import .database_download
from .file_download import download_text as _download_text
from .exceptions import BaseError, BaseWarning, InvalidParameterError, NoInternetError, OldPackageVersionWarning

def list_datasets():
    """List all available datasets."""
    dataset_list_url = "https://bcm.box.com/shared/static/v24wiuyy73rhv8jn44tyjbs0al8wa6oe.csv"
    try:
        dataset_list_text = _download_text(dataset_list_url)
    except NoInternetError:
        raise NoInternetError("Insufficient internet to download available dataset info. Check your internet connection.") from None
    return pd.read_csv(io.StringIO(dataset_list_text), header=0)
